Type: Package
Package: getCRUCLdata
Title: 'CRU' 'CL' v. 2.0 Climatology Client
Version: 2.0.0
Authors@R: 
    person("Adam H.", "Sparks", , "adamhsparks@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-0061-8359"))
Description: Provides functions that automate downloading and importing
    University of East Anglia Climate Research Unit ('CRU') 'CL' v. 2.0
    climatology data, facilitates the calculation of minimum temperature
    and maximum temperature and formats the data into a data.table object
    or a 'terra' 'SpatRaster' object.  'CRU' 'CL' v. 2.0 data are a
    gridded climatology of 1961-1990 monthly means released in 2002 and
    cover all land areas (excluding Antarctica) at 10 arc minutes
    (0.1666667 degree) resolution.  For more information see the
    description of the data provided by the University of East Anglia
    Climate Research Unit,
    <https://crudata.uea.ac.uk/cru/data/hrg/tmc/readme.txt>.
License: MIT + file LICENSE
URL: https://codeberg.org/ropensci/getCRUCLdata,
        https://docs.ropensci.org/getCRUCLdata/
BugReports: https://codeberg.org/ropensci/getCRUCLdata/issues
Depends: R (>= 4.1.0)
Imports: brio, cli, data.table, fs, httr2, rlang, terra
Suggests: knitr, R.utils, rmarkdown, roxygen2, roxyglobals, spelling,
        testthat, withr
VignetteBuilder: knitr
ByteCompile: TRUE
Config/Needs/build: moodymudskipper/devtag
Config/roxygen2/version: 8.0.0
Config/roxyglobals/filename: globals.R
Config/roxyglobals/unique: FALSE
Config/roxylint: list(linters = roxylint::tidy)
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
Language: en-US
X-schema.org-applicationCategory: Tools
X-schema.org-isPartOf: https://ropensci.org
X-schema.org-keywords: anglia-cru, climate-data, cru-cl2, temperature,
        rainfall, elevation, data-access, wind, relative-humidity,
        solar-radiation, diurnal-temperature, frost
NeedsCompilation: no
Packaged: 2026-05-16 11:48:48 UTC; adamsparks
Author: Adam H. Sparks [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0061-8359>)
Maintainer: Adam H. Sparks <adamhsparks@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-20 09:10:02 UTC
Built: R 4.5.2; ; 2026-05-21 23:31:29 UTC; unix

# Generated by roxygen2: do not edit by hand

export(read_cru_dt)
export(read_cru_rast)
importFrom(brio,write_file_raw)
importFrom(cli,cli_abort)
importFrom(data.table,"%notin%")
importFrom(data.table,":=")
importFrom(data.table,.BY)
importFrom(data.table,.EACHI)
importFrom(data.table,.GRP)
importFrom(data.table,.I)
importFrom(data.table,.N)
importFrom(data.table,.NGRP)
importFrom(data.table,.SD)
importFrom(data.table,data.table)
importFrom(fs,dir_create)
importFrom(fs,path_dir)
importFrom(fs,path_temp)
importFrom(httr2,req_cache)
importFrom(httr2,req_error)
importFrom(httr2,req_headers)
importFrom(httr2,req_options)
importFrom(httr2,req_perform)
importFrom(httr2,req_retry)
importFrom(httr2,req_user_agent)
importFrom(httr2,request)
importFrom(httr2,resp_body_raw)
importFrom(httr2,resp_status)

# getCRUCLdata: Use and Explore CRU CL v. 2.0 Climatology Elements in R

<!-- badges: start -->

[![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/) [![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.466812.svg)](https://doi.org/10.5281/zenodo.466812) [![JOSS status](http://joss.theoj.org/papers/10.21105/joss.00230/status.svg)](https://joss.theoj.org/papers/10.21105/joss.00230) [![](https://badges.ropensci.org/96_status.svg)](https://github.com/ropensci/software-review/issues/96)

<!-- badges: end -->

## Introduction to {getCRUCLdata}

The {getCRUCLdata} package provides functions that automate importing [CRU CL v. 2.0 climatology data](https://crudata.uea.ac.uk/~timm/grid/CRU_CL_2_0.html) into R, facilitates the calculation of minimum temperature and maximum temperature, and formats the data into a [data.table](https://CRAN.R-project.org/package=data.table) or a [terra::SpatRaster](https://CRAN.R-project.org/package=terra) object.

CRU CL v. 2.0 data are a gridded climatology of 1961-1990 monthly means released in 2002 and cover all land areas (excluding Antarctica) at 10 arc minutes (0.1666667 degree) resolution.
 For more information see the description of the data provided by the University of East Anglia Climate Research Unit (CRU), <https://crudata.uea.ac.uk/cru/data/hrg/tmc/readme.txt>.


## Changes to original CRU CL v. 2.0 data

This package automatically converts elevation values from kilometres to metres.

This package crops all spatial outputs to an extent of ymin = -60, ymax = 85, xmin = -180, xmax = 180.
Note that the original wind data include land area for parts of Antarctica, and so these are omitted.
There are also some elevation values around the Dead Sea area that do not have corresponding climate values and so these are removed.
What is left is a complete suite of data for climate and elevation data in all grid cells that are provided, but not *all* the data that are in the original datasets.

## Quick Start

### Install

{getCRUCLdata} is not available from CRAN.
You can install it from the rOpenSci Universe as follows.

``` r
# Enable the rOpenSci R-universe
options(
  repos = c(
    rOpenSci = "https://ropensci.r-universe.dev",
    CRAN = "https://cloud.r-project.org"
  )
)
# Install the package
install.packages("getCRUCLdata")
```

## Documentation

For complete documentation see the package website: <https://docs.ropensci.org/getCRUCLdata/>.

## Meta

### CRU CL v. 2.0 Reference and Abstract

> Mark New (1,\*), David Lister (2), Mike Hulme (3), Ian Makin (4)

> A high-resolution data set of surface climate over global land areas Climate Research, 2000, Vol 21, pg 1-25

> 1. School of Geography and the Environment, University of Oxford, Mansfield Road, Oxford OX1 3TB, United Kingdom
> 2. Climatic Research Unit, and (3) Tyndall Centre for Climate Change Research, both at School of Environmental Sciences, University of East Anglia, Norwich NR4 7TJ, United Kingdom
> 3. International Water Management Institute, PO Box 2075, Colombo, Sri Lanka

> **ABSTRACT:** We describe the construction of a 10-minute latitude/longitude data set of mean monthly surface climate over global land areas, excluding Antarctica. The climatology includes 8 climate elements - precipitation, wet-day frequency, temperature, diurnal temperature range, relative humidity, sunshine duration, ground frost frequency and windspeed - and was interpolated from a data set of station means for the period centred on 1961 to 1990. Precipitation was first defined in terms of the parameters of the Gamma distribution, enabling the calculation of monthly precipitation at any given return period. The data are compared to an earlier data set at 0.5 degrees latitude/longitude resolution and show added value over most regions. The data will have many applications in applied climatology, biogeochemical modelling, hydrology and agricultural meteorology and are available through the School of Geography Oxford (<https://www.geog.ox.ac.uk/>), the International Water Management Institute “World Water and Climate Atlas” (<https://www.iwmi.org/>) and the Climatic Research Unit (<https://www.uea.ac.uk/groups-and-centres/climatic-research-unit>).

### Other

- Please [report any issues or bugs](https://codeberg.org/ropensci/getCRUCLdata/issues).

- License: MIT

- Get citation information for *getCRUCLdata* in R typing `citation(package = "getCRUCLdata")`

- Please note that this package is released with a [Contributor Code of Conduct](https://ropensci.org/code-of-conduct/).
By contributing to this project, you agree to abide by its terms.

#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @importFrom data.table :=
#' @importFrom data.table .BY
#' @importFrom data.table .EACHI
#' @importFrom data.table .GRP
#' @importFrom data.table .I
#' @importFrom data.table .N
#' @importFrom data.table .NGRP
#' @importFrom data.table .SD
#' @importFrom data.table data.table
#' @importFrom data.table %notin%
## usethis namespace: end
NULL

#' For `elv`: produces a single-layer raster with bad coordinates masked to NA.
#' For all other variables: produces 12 layers (one per month), named
#' `<varname>_<month>`.
#'
#' @param dt A tidy data.table with columns: lat, lon, value, and (for
#'   non-elv variables) month.
#' @param varname Character scalar: variable identifier (e.g., "tmp", "pre",
#'  "elv").
#'
#' @return A [terra::rast] - one layer for elv, 12 layers for all others.
#' @autoglobal
#' @dev

.dt_to_rast <- function(dt, varname) {
  wrld <- .cru_template_rast()

  # --- elevation: single layer, bad coordinates masked ----------------------
  if (varname == "elv") {
    value_col <- if ("elv" %in% names(dt)) "elv" else "value"
    xy <- cbind(dt$lon, dt$lat)
    cell <- terra::cellFromXY(wrld, xy)
    wrld[cell] <- dt[[value_col]]
    wrld <- .remove_bad_cells_rast(wrld)
    names(wrld) <- "elv"
    return(wrld)
  }

  # --- all other variables: 12 monthly layers -------------------------------
  value_col <- if ("value" %in% names(dt)) "value" else varname

  rast_list <- lapply(.cru_month_names, function(m) {
    r <- wrld
    dtm <- dt[month == m]
    cells <- terra::cellFromXY(r, cbind(dtm$lon, dtm$lat))
    r[cells] <- dtm[[value_col]]
    r
  })

  names(rast_list) <- sprintf("%s_%s", varname, .cru_month_names)
  return(terra::rast(rast_list))
}


#' Mask bad cells in a SpatRaster
#'
#' Sets the package-level `.bad_coords` cells to NA. Used exclusively for the
#' elevation layer.
#'
#' @param r A [terra::rast] to modify.
#'
#' @return The modified [terra::rast].
#' @autoglobal
#' @dev
.remove_bad_cells_rast <- function(r) {
  cells <- terra::cellFromXY(r, as.matrix(.bad_coords[, list(lon, lat)]))
  cells <- unique(stats::na.omit(cells))
  r[cells] <- NA
  return(r)
}

test_that(".dt_to_rast() returns a terra SpatRaster", {
  # Create minimal test data
  dt <- data.table(
    lat = c(-60, -60, 0, 0, 60, 60),
    lon = c(-120, 120, -120, 120, -120, 120),
    month = rep(c("jan", "feb"), 3),
    value = c(10, 15, 20, 25, 5, 8)
  )

  result <- .dt_to_rast(dt, "tmp")

  expect_s4_class(result, "SpatRaster")
})

test_that(".dt_to_rast() creates 12 layers (one per month)", {
  dt <- data.table(
    lat = c(-60, 0, 60),
    lon = c(-120, 0, 120),
    month = c("jan", "feb", "mar"),
    value = c(10, 20, 30)
  )

  result <- .dt_to_rast(dt, "pre")

  expect_identical(terra::nlyr(result), 12)
})

test_that(".dt_to_rast() applies varname correctly to layer names", {
  dt <- data.table(
    lat = c(-60, 0),
    lon = c(-120, 120),
    month = c("jan", "feb"),
    value = c(10, 20)
  )

  result <- .dt_to_rast(dt, "tmp")

  expected_names <- paste0(
    "tmp_",
    c(
      "jan",
      "feb",
      "mar",
      "apr",
      "may",
      "jun",
      "jul",
      "aug",
      "sep",
      "oct",
      "nov",
      "dec"
    )
  )
  expect_equal(names(result), expected_names)
})

test_that(".dt_to_rast() uses correct CRU grid dimensions", {
  dt <- data.table(
    lat = c(-60, 0, 60),
    lon = c(-120, 0, 120),
    month = c("jan", "feb", "mar"),
    value = c(10, 20, 30)
  )

  result <- .dt_to_rast(dt, "tmp")

  expect_identical(terra::nrow(result), 930)
  expect_identical(terra::ncol(result), 2160)
  expect_identical(terra::ext(result)$xmin, -180, ignore_attr = TRUE)
  expect_identical(terra::ext(result)$xmax, 180, ignore_attr = TRUE)
  expect_identical(terra::ext(result)$ymin, -65, ignore_attr = TRUE)
  expect_identical(terra::ext(result)$ymax, 90, ignore_attr = TRUE)
})

test_that(".dt_to_rast() correctly places values at cell coordinates", {
  dt <- data.table(
    lat = c(0, 0),
    lon = c(0, 180),
    month = c("jan", "jan"),
    value = c(100, 200)
  )

  result <- .dt_to_rast(dt, "tmp")

  # Extract January layer
  jan_layer <- result[[1]]

  # Get cell indices for coordinates
  xy_0_0 <- cbind(0, 0)
  xy_180_0 <- cbind(180, 0)
  cell_0_0 <- terra::cellFromXY(jan_layer, xy_0_0)
  cell_180_0 <- terra::cellFromXY(jan_layer, xy_180_0)

  # Values should be in those cells
  expect_identical(terra::values(jan_layer)[cell_0_0], 100)
  expect_identical(terra::values(jan_layer)[cell_180_0], 200)
})

test_that(".dt_to_rast() handles all 12 months correctly", {
  # Create data with all months
  months <- c(
    "jan",
    "feb",
    "mar",
    "apr",
    "may",
    "jun",
    "jul",
    "aug",
    "sep",
    "oct",
    "nov",
    "dec"
  )
  dt <- data.table(
    lat = rep(0, 12),
    lon = rep(0, 12),
    month = months,
    value = seq_len(12)
  )

  result <- .dt_to_rast(dt, "tmp")

  # Verify all months have data
  for (i in seq_along(months)) {
    layer <- result[[i]]
    xy <- cbind(0, 0)
    cell <- terra::cellFromXY(layer, xy)
    expect_identical(terra::values(layer)[cell], as.numeric(i))
  }
})

test_that(".dt_to_rast() throws error when required columns are missing", {
  # Missing 'value' column
  dt_bad <- data.table(
    lat = c(-60, 0),
    lon = c(-120, 120),
    month = c("jan", "feb")
  )

  expect_error(.dt_to_rast(dt_bad, "tmp"))
})

test_that(".dt_to_rast() throws error when month column is missing", {
  dt_bad <- data.table(
    lat = c(-60, 0),
    lon = c(-120, 120),
    value = c(10, 20)
  )

  expect_error(.dt_to_rast(dt_bad, "tmp"))
})

test_that(".dt_to_rast() handles empty month subsets gracefully", {
  # Data with some months missing
  dt <- data.table(
    lat = c(0, 0),
    lon = c(0, 0),
    month = c("jan", "jan"), # Only January data
    value = c(10, 20)
  )

  result <- .dt_to_rast(dt, "tmp")

  # Should still return 12 layers
  expect_identical(terra::nlyr(result), 12)

  # Other months should be all NA
  feb_layer <- result[[2]]
  expect_true(all(is.na(terra::values(feb_layer))))
})

test_that(".dt_to_rast() works with different variable names", {
  dt <- data.table(
    lat = 0,
    lon = 0,
    month = "jan",
    value = 25
  )

  result_tmp <- .dt_to_rast(dt, "tmp")
  result_pre <- .dt_to_rast(dt, "pre")

  expect_true(grepl("^tmp_", names(result_tmp)[1]))
  expect_true(grepl("^pre_", names(result_pre)[1]))
})

test_that(".dt_to_rast() handles boundary coordinates", {
  dt <- data.table(
    lat = c(-65, 90, -65, 90),
    lon = c(-180, 180, 180, -180),
    month = rep("jan", 4),
    value = c(1, 2, 3, 4)
  )

  result <- .dt_to_rast(dt, "tmp")

  # Should not error and should produce valid raster
  expect_s4_class(result, "SpatRaster")
  expect_identical(terra::nlyr(result), 12)
})

test_that(".dt_to_rast() produces correct data type in output", {
  dt <- data.table(
    lat = 0,
    lon = 0,
    month = "jan",
    value = 42.5
  )

  result <- .dt_to_rast(dt, "tmp")

  # Check that numeric values are preserved
  jan_layer <- result[[1]]
  xy <- cbind(0, 0)
  cell <- terra::cellFromXY(jan_layer, xy)
  value <- terra::values(jan_layer)[cell]

  expect_type(value, "double")
  expect_equal(value, 42.5, tolerance = 1e-6)
})

#' Ensure gzip support is available for reading .gz files
#'
#' Internal helper used by both .tidy_dt() and .create_rast().
#'
#' Aborts if .gz files are present and neither R.utils nor system gzip
#' is available.
#'
#' @param files Character vector of file paths.
#' @returns An invisible `TRUE` if support is found.
#' @dev

.check_gzip_support <- function(files) {
  if (!is.character(files) || !any(endsWith(files, ".gz"))) {
    return(invisible(TRUE))
  }

  if (
    !requireNamespace("R.utils", quietly = TRUE) &&
      Sys.which("gzip") == ""
  ) {
    cli::cli_abort(
      c(
        x = "Cannot read compressed `.gz` files.",
        i = "Package {.pkg R.utils} is not installed.",
        i = "System {.code gzip} command not found.",
        i = "Install {.pkg R.utils} with {.code install.packages('R.utils')}."
      )
    )
  }

  invisible(TRUE)
}

#' Filter CRU file names based on selected variables
#'
#' @param vars Named logical vector.
#'
#' @returns Character vector of file names.
#' @autoglobal
#' @dev

.filter_files <- function(vars) {
  vars <- .normalize_vars(vars)

  file_vars <- intersect(names(vars)[vars], names(.cru_files))
  .cru_files[file_vars]
}

#' Validate and filter CRU file selections
#'
#' @param vars Named logical vector.
#' @param x Optional path.
#'
#' @returns Character vector of file names.
#' @dev
.validate_filter_files <- function(vars, x) {
  .check_vars(vars)
  .filter_files(vars) # just names, no paths
}

.onAttach <- function(libname, pkgname) {
  if (!requireNamespace("R.utils", quietly = TRUE)) {
    cli::cli_inform(
      c(
        x = "Package {.pkg R.utils} is not installed.",
        i = "Reading `.gz` files with {.fn data.table::fread} may fail.",
        i = "Install it with {.code install.packages('R.utils')}."
      )
    )
  }
}

# Helper to locate fixture files
fixture_path <- function(name) {
  testthat::test_path("fixtures", name)
}

copy_fixture_to_temp <- function(name) {
  dest <- file.path(tempdir(), name)
  fs::file_copy(fixture_path(name), dest, overwrite = TRUE)
  dest
}

test_that(".tidy_dt merges multiple CRU files correctly", {
  pre <- copy_fixture_to_temp("grid_10min_pre.dat.gz")
  tmp <- copy_fixture_to_temp("grid_10min_tmp.dat.gz")

  vars <- c(
    pre = TRUE,
    pre_cv = FALSE,
    tmp = TRUE,
    dtr = FALSE,
    reh = FALSE,
    tmn = FALSE,
    tmx = FALSE,
    sunp = FALSE,
    frs = FALSE,
    wnd = FALSE,
    elv = FALSE
  )

  tidy_dt <- .tidy_dt(vars, c(pre, tmp))

  expect_s3_class(tidy_dt, "data.table")
  expect_true(all(c("lat", "lon", "month", "pre", "tmp") %in% names(tidy_dt)))
  expect_identical(nrow(tidy_dt), 10L * 12L)
})

test_that(".create_dt computes derived variables correctly", {
  pre_file <- copy_fixture_to_temp("grid_10min_pre.dat.gz")
  tmp_file <- copy_fixture_to_temp("grid_10min_tmp.dat.gz")

  vars <- c(
    pre = TRUE,
    pre_cv = FALSE,
    tmp = TRUE,
    dtr = FALSE,
    reh = FALSE,
    tmn = FALSE,
    tmx = FALSE,
    sunp = FALSE,
    frs = FALSE,
    wnd = FALSE,
    elv = FALSE
  )

  # IMPORTANT: pass file paths, not tidy data
  dtv <- .create_dt(vars, c(pre_file, tmp_file))

  expect_s3_class(dtv, "data.table")
  expect_true(all(c("lat", "lon", "month", "pre", "tmp") %in% names(dtv)))
})

test_that(".tidy_dt merges fixture data correctly (parallel-safe)", {
  pre <- copy_fixture_to_temp("grid_10min_pre.dat.gz")
  tmp <- copy_fixture_to_temp("grid_10min_tmp.dat.gz")

  vars <- c(
    pre = TRUE,
    tmp = TRUE,
    pre_cv = FALSE,
    dtr = FALSE,
    reh = FALSE,
    tmn = FALSE,
    tmx = FALSE,
    sunp = FALSE,
    frs = FALSE,
    wnd = FALSE,
    elv = FALSE
  )

  dt <- .tidy_dt(vars, c(pre, tmp))

  expect_s3_class(dt, "data.table")
  expect_true(all(c("lat", "lon", "month", "pre", "tmp") %in% names(dt)))
})

test_that(".create_dt computes derived variables correctly", {
  pre <- copy_fixture_to_temp("grid_10min_pre.dat.gz")
  tmp <- copy_fixture_to_temp("grid_10min_tmp.dat.gz")

  # Create a fake dtr fixture by copying tmp (just for test structure)
  dtr <- copy_fixture_to_temp("grid_10min_tmp.dat.gz")

  vars <- c(
    pre = TRUE,
    tmp = TRUE,
    dtr = FALSE,
    tmn = FALSE,
    tmx = FALSE,
    pre_cv = FALSE,
    reh = FALSE,
    sunp = FALSE,
    frs = FALSE,
    wnd = FALSE,
    elv = FALSE
  )

  dt_test <- .create_dt(vars, c(pre, tmp))

  expect_true("tmp" %in% names(dt_test))
  expect_true("pre" %in% names(dt_test))
})

#' Read and merge CRU files into a tidy data.table
#'
#' @param vars Named logical vector.
#' @param files Character vector of CRU file paths.
#'
#' @returns A merged data.table.
#' @dev

.tidy_dt <- function(vars, files) {
  .check_gzip_support(files)

  dt_list <- lapply(files, .read_local_files, vars = vars)

  varnames <- sub(
    "^grid_10min_([a-z0-9_]+)\\.dat\\.gz$",
    "\\1",
    fs::path_file(files)
  )
  names(dt_list) <- varnames

  # Separate elevation (no month column)
  elv_dt <- NULL
  if ("elv" %in% names(dt_list)) {
    elv_dt <- dt_list[["elv"]]
    dt_list[["elv"]] <- NULL
  }

  monthly <- Filter(function(dt) "month" %in% names(dt), dt_list)

  if (length(monthly) == 0L) {
    return(if (!is.null(elv_dt)) elv_dt[] else data.table::data.table())
  }

  # All monthly tables share identical lat/lon/month rows — key and join
  lapply(monthly, data.table::setkey, lat, lon, month)
  merged <- Reduce(function(x, y) x[y], monthly)

  # Join elevation
  if (!is.null(elv_dt)) {
    data.table::setkey(elv_dt, lat, lon)
    data.table::setkey(merged, lat, lon)
    merged <- elv_dt[merged]
  }

  merged[]
}

#' Create a terra SpatRaster object of CRU CL v. 2.0 climatology elements
#'
#' Create a [terra::SpatRaster] object or list of objects from \acronym{CRU} CL
#'  2.0 data.
#'
#' @inherit read_cru_dt
#'
#' @examplesIf interactive()
#' # Create a data frame of temperature from locally available files in the
#' # tempdir() directory.
#' library(fs)
#'
#' f <- path(path_temp(), "grid_10min_tmp.dat.gz")
#'
#' download.file(
#'   url = "https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_tmp.dat.gz",
#'   destfile = f
#' )
#'
#'
#' cru_tmp <- read_cru_rast(tmp = TRUE, x = f)
#'
#' cru_tmp
#'
#' # or downloading directly from the CRU server
#'
#' cru_tmp <- read_cru_rast(tmp = TRUE)
#'
#' cru_tmp
#'
#' @returns A [terra::SpatRaster] object.
#' @export

read_cru_rast <- function(
  pre = FALSE,
  pre_cv = FALSE,
  rd0 = FALSE,
  tmp = FALSE,
  dtr = FALSE,
  reh = FALSE,
  tmn = FALSE,
  tmx = FALSE,
  sunp = FALSE,
  frs = FALSE,
  wnd = FALSE,
  elv = FALSE,
  x = NULL
) {
  vars <- c(
    pre = pre,
    pre_cv = pre_cv,
    rd0 = rd0,
    tmp = tmp,
    dtr = dtr,
    reh = reh,
    tmn = tmn,
    tmx = tmx,
    sunp = sunp,
    frs = frs,
    wnd = wnd,
    elv = elv
  )

  files <- .validate_filter_files(vars, x)
  resolved <- .file_handling(vars, files, x)

  .create_rast(resolved$vars, resolved$files)
}

test_that("read_cru_rast builds a SpatRaster from fixtures", {
  local_d <- withr::local_tempdir()

  fs::dir_copy(
    test_path("fixtures"),
    local_d,
    overwrite = TRUE
  )

  r <- read_cru_rast(pre = TRUE, tmp = TRUE, x = local_d)

  expect_s4_class(r, "SpatRaster")
  expect_true(any(grepl("pre", names(r))))
  expect_true(any(grepl("tmp", names(r))))
})

#' Resolve CRU variable flags and input file paths
#'
#' Internal helper used by higher-level CRU readers to convert user-facing
#' arguments (e.g., `pre = TRUE`, `tmp = TRUE`, `x = <dir>`) into a standardised
#' structure containing:
#'
#' * a named logical vector of requested CRU variables (`vars`)
#' * a character vector of file paths (`files`)
#'
#' If `x` is a directory, all matching CRU files (`grid_10min_*.dat.gz`) inside
#' that directory are returned.
#' If `x` is one or more file paths, they are returned unchanged.
#'
#' This function does **not** read or validate file contents; it only resolves
#' which files should be processed downstream.
#'
#' @param pre Logical. Whether precipitation (`pre`) is requested.
#' @param tmp Logical. Whether temperature (`tmp`) is requested.
#' @param x Character. Either a directory containing CRU files or one or more
#'   CRU `.dat.gz` file paths.
#' @param ... Ignored. Allows future expansion.
#'
#' @return A list with two elements:
#' \describe{
#'   \item{vars}{Named logical vector of CRU variable flags.}
#'   \item{files}{Character vector of resolved CRU file paths.}
#' }
#'
#' @dev

.resolve_cru_inputs <- function(pre = FALSE, tmp = FALSE, x = NULL, ...) {
  vars <- c(
    pre = pre,
    tmp = tmp,
    dtr = FALSE,
    tmn = FALSE,
    tmx = FALSE,
    pre_cv = FALSE,
    reh = FALSE,
    sunp = FALSE,
    frs = FALSE,
    wnd = FALSE,
    elv = FALSE
  )

  # If x is a directory → list files
  files <- c(
    fs::dir_ls(x[fs::is_dir(x)], regexp = "grid_10min_.*\\.dat\\.gz$"),
    x[!fs::is_dir(x)]
  )

  return(list(vars = vars, files = files))
}

#' Read a single CRU file into a tidy data.table
#'
#' @param file CRU .dat.gz file path or a directory containing one such file.
#' @param vars Named logical vector.
#'
#' @returns A tidy data.table.
#' @autoglobal
#' @dev

.read_local_files <- function(file, vars) {
  # 1. Directory -> read and merge all matching files within
  if (fs::is_dir(file)) {
    paths <- fs::dir_ls(
      file,
      regexp = "grid_10min_.*\\.dat\\.gz$",
      recurse = FALSE
    )
    if (!length(paths)) {
      cli::cli_abort("No CRU .dat.gz files found in {.path {file}}.")
    }
    return(.tidy_dt(vars, paths))
  }

  # 2. Single file -> parse into tidy dt
  x <- data.table::fread(file, header = FALSE)
  n <- ncol(x)

  # Derive varname from filename
  varname <- sub(
    "^grid_10min_([a-z0-9_]+)\\.dat\\.gz$",
    "\\1",
    fs::path_file(file)
  )

  # 14-column monthly variable
  if (n == 14L) {
    data.table::setnames(x, c("lat", "lon", .cru_month_names))
    x <- data.table::melt(
      x,
      id.vars = c("lat", "lon"),
      variable.name = "month",
      value.name = varname
    )
    return(x[])
  }

  # 26-column pre + pre_cv
  if (n == 26L) {
    pre_dt <- x[, 1:14]
    data.table::setnames(pre_dt, c("lat", "lon", .cru_month_names))
    pre_dt <- data.table::melt(
      pre_dt,
      id.vars = c("lat", "lon"),
      variable.name = "month",
      value.name = "pre"
    )

    if (isFALSE(vars["pre_cv"])) {
      return(pre_dt[])
    }

    cv_dt <- x[, c(1L, 2L, 15:26)]
    data.table::setnames(cv_dt, c("lat", "lon", .cru_month_names))
    cv_dt <- data.table::melt(
      cv_dt,
      id.vars = c("lat", "lon"),
      variable.name = "month",
      value.name = "pre_cv"
    )

    data.table::setkey(pre_dt, lat, lon, month)
    data.table::setkey(cv_dt, lat, lon, month)
    pre_dt[cv_dt, pre_cv := i.pre_cv]
    return(pre_dt[])
  }

  # 3-column elevation
  if (n == 3L) {
    data.table::setnames(x, c("lat", "lon", "elv"))
    x[, elv := elv * 1000L]
    return(x[])
  }

  cli::cli_abort("Unexpected file format in {.path {file}}.")
}

test_that(".read_local_files lists local files", {
  vars <- c(
    pre = TRUE,
    pre_cv = FALSE,
    tmp = FALSE,
    dtr = FALSE,
    reh = FALSE,
    tmn = FALSE,
    tmx = FALSE,
    sunp = FALSE,
    frs = FALSE,
    wnd = FALSE,
    elv = FALSE
  )

  # Test 1: Directory with files (test_path already points to fixtures)
  fixture_dir <- test_path("fixtures")
  dtvar <- .read_local_files(fixture_dir, vars)
  expect_s3_class(dtvar, "data.table")
  expect_true(all(c("lat", "lon", "month") %in% names(dtvar)))

  # Test 2: Single file directly
  pre_file <- test_path("fixtures", "grid_10min_pre.dat.gz")
  dtvar2 <- .read_local_files(file = pre_file, vars = vars)
  expect_s3_class(dtvar2, "data.table")
  expect_true(all(c("lat", "lon", "month", "pre") %in% names(dtvar2)))
})

#' Internal helper for creating terra layers
#'
#' @param file CRU .dat.gz file path.
#' @param wrld Empty `terra::rast` template.
#' @param vars Named logical vector of CRU variable selections.
#'
#' @returns A [terra::rast] object for one variable.
#' @dev
.make_rast <- function(file, wrld, vars, varname) {
  wvar <- data.table::fread(file, header = FALSE)
  cells <- terra::cellFromXY(wrld, wvar[, c(2L, 1L)])
  n <- ncol(wvar)

  # Helper: build a list of 12 monthly layers from column indices
  build_monthly <- function(cols, prefix = NULL) {
    layers <- lapply(seq_along(cols), function(i) {
      r <- terra::rast(wrld)
      r[cells] <- wvar[[cols[i]]]
      r
    })
    names(layers) <- if (is.null(prefix)) {
      .cru_month_names
    } else {
      paste0(prefix, .cru_month_names)
    }
    layers
  }

  # Helper: build a single-layer raster
  build_single <- function(col, name) {
    r <- terra::rast(wrld)
    r[cells] <- wvar[[col]]
    names(r) <- name
    r
  }

  # --- Case 1: Standard 12-month variables (14 columns) ---
  if (n == 14L) {
    layers <- build_monthly(3:14)
    names(layers) <- paste0(varname, "_", .cru_month_names)
    return(terra::rast(layers))
  }

  # --- Case 2: pre + pre_cv (26 columns) ---
  if (n == 26L) {
    pre_layers <- build_monthly(3:14, "pre_")
    if (!isTRUE(vars["pre_cv"])) {
      return(terra::rast(pre_layers))
    }
    cv_layers <- build_monthly(15:26, "pre_cv_")
    return(terra::rast(c(pre_layers, cv_layers)))
  }

  # --- Case 3: elevation (3 columns) ---
  if (n == 3L) {
    r <- build_single(3L, "elv")
    r <- r * 1000L # convert from km to m
    r <- .remove_bad_cells_rast(r)
    return(r)
  }

  cli::cli_abort("Unexpected file format in {.var file}.")
}


#' Create terra rast objects
#'
#' @param vars Named logical vector.
#' @param files Character vector of CRU file paths.
#'
#' @returns A terra::rast object.
#' @autoglobal
#' @dev
.create_rast <- function(vars, files) {
  # 1. Expand directory -> file paths
  if (is.character(files) && length(files) == 1L && fs::is_dir(files)) {
    files <- fs::dir_ls(files, regexp = "grid_10min_.*\\.dat\\.gz$")
  }

  # 2. Check gzip support for character paths
  if (is.character(files)) {
    .check_gzip_support(files)
    if (!length(files)) cli::cli_abort("No CRU raster files found.")
  }

  # 3. Handle pre-built SpatRaster or tidy data.table list inputs
  if (all(vapply(files, inherits, logical(1), what = "SpatRaster"))) {
    return(terra::crop(terra::rast(files), .cru_extent()))
  }

  if (is.list(files) && !is.character(files)) {
    rast_list <- lapply(names(files), function(v) {
      .dt_to_rast(files[[v]], varname = v)
    })
    return(terra::crop(terra::rast(rast_list), .cru_extent()))
  }

  # 4. Build rasters from raw CRU files
  wrld <- .cru_template_rast()
  varnames <- .cru_varname(files)

  rast_list <- lapply(seq_along(files), function(i) {
    .make_rast(
      file = files[[i]],
      wrld = wrld,
      vars = vars,
      varname = varnames[[i]]
    )
  })
  names(rast_list) <- varnames

  if (!all(vapply(rast_list, inherits, logical(1), what = "SpatRaster"))) {
    cli::cli_abort("One or more CRU rasters failed to build.")
  }

  # 5. Derived variables (tmn / tmx from tmp +/- 0.5 * dtr)
  if (isTRUE(vars["tmn"]) && all(c("tmp", "dtr") %in% names(rast_list))) {
    rast_list$tmn <- rast_list$tmp - 0.5 * rast_list$dtr
  }
  if (isTRUE(vars["tmx"]) && all(c("tmp", "dtr") %in% names(rast_list))) {
    rast_list$tmx <- rast_list$tmp + 0.5 * rast_list$dtr
  }

  # Drop tmp / dtr if they were only needed as intermediates
  if ((isTRUE(vars["tmn"]) || isTRUE(vars["tmx"])) && isFALSE(vars["tmp"])) {
    rast_list$tmp <- NULL
  }
  if ((isTRUE(vars["tmn"]) || isTRUE(vars["tmx"])) && isFALSE(vars["dtr"])) {
    rast_list$dtr <- NULL
  }

  # Crop to CRU CL 2.0 extent: ymin = -60, ymax = 85
  terra::crop(terra::rast(rast_list), .cru_extent())
}


#' CRU 10-minute global raster template
#'
#' Shared baseline grid (930 x 2160, lat -65 to 90, lon -180 to 180).
#' All values initialised to NA_real_.
#'
#' @return A single-layer terra::rast.
#' @dev
.cru_template_rast <- function() {
  r <- terra::rast(
    nrows = 930,
    ncols = 2160,
    ymin = -65,
    ymax = 90,
    xmin = -180,
    xmax = 180
  )
  r[] <- NA_real_
  return(r)
}


#' CRU CL 2.0 output crop extent
#'
#' Crops all raster outputs to ymin = -60, ymax = 85, xmin = -180, xmax = 180,
#' matching the stated coverage of the CRU CL 2.0 dataset in the package's
#' README. This crops the wind data that cover the Antarctic region.
#'
#' @return A terra::ext object.
#' @dev
.cru_extent <- function() {
  terra::ext(-180, 180, -60, 85)
}


#' Extract CRU variable names from file paths
#'
#' Strips the `grid_10min_` prefix and `.dat.gz` suffix from CRU filenames,
#' returning the bare variable identifier (e.g. "pre", "pre_cv", "elv").
#'
#' @param files Character vector of CRU `.dat.gz` file paths.
#' @return Character vector of variable names.
#' @dev
.cru_varname <- function(files) {
  base <- fs::path_file(files)
  file_stem <- sub("\\.dat\\.gz$", "", base)
  sub("^grid_10min_", "", file_stem)
}

# vignettes that depend on internet access need to be precompiled and take a
# while to run
devtools::install() # ensure we're using latest version

knitr::knit("vignettes/getCRUCLdata.Rmd.orig", "vignettes/getCRUCLdata.Rmd")

# remove file path such that vignettes will build with figures
replace <- readLines("vignettes/getCRUCLdata.Rmd")
replace <- gsub("<img src=\"vignettes/", "<img src=\"", replace)
fileConn <- file("vignettes/getCRUCLdata.Rmd")
writeLines(replace, fileConn)
close(fileConn)

# build vignettes
devtools::build_vignettes()

# move resource files to /doc
resources <-
  list.files("vignettes/", pattern = ".png$", full.names = TRUE)
file.copy(from = resources, to = "doc", overwrite = TRUE)

#' Use httr2 to fetch a file with retries
#'
#' Downloads a file with retry logic and optional progress display.
#' The downloaded file is written to `dest`.
#'
#' @param url Character. The full URL to download.
#' @param dest Character. File path where the downloaded file will be written.
#' @param .max_tries Integer. Number of retry attempts.
#'
#' @returns Invisibly returns `dest` for convenience.
#' @dev Internal function, not user-facing.
#' @importFrom fs dir_create path_dir path_temp
#' @importFrom httr2 request req_user_agent req_headers req_retry req_cache
#' @importFrom httr2 req_error req_options req_perform resp_status resp_body_raw
#' @importFrom brio write_file_raw
#' @importFrom cli cli_abort
.retry_download <- function(url, dest, .max_tries = 3L) {
  fs_dir_create(fs_path_dir(dest))

  req <- req_request(url) |>
    req_user_agent("getCRUCLdata") |>
    req_headers(
      "Accept-Encoding" = "identity",
      "Connection" = "Keep-Alive"
    ) |>
    req_retry(max_tries = .max_tries) |>
    req_cache(path = fs_path_temp()) |>
    req_error(is_error = function(resp) FALSE) |>
    req_options()

  resp <- req_perform(req)
  status <- resp_status(resp)

  if (status >= 400L) {
    cli::cli_abort("Failed to download {.url {url}} (HTTP {status}).")
  }

  write_file_raw(resp_body_raw(resp), path = dest)
  invisible(dest)
}

#' @noRd
fs_dir_create <- function(...) fs::dir_create(...)

#' @noRd
fs_path_dir <- function(...) fs::path_dir(...)

#' @noRd
fs_path_temp <- function(...) fs::path_temp(...)

#' @noRd
req_request <- function(...) httr2::request(...)

#' @noRd
req_user_agent <- function(...) httr2::req_user_agent(...)

#' @noRd
req_headers <- function(...) httr2::req_headers(...)

#' @noRd
req_retry <- function(...) httr2::req_retry(...)

#' @noRd
req_cache <- function(...) httr2::req_cache(...)

#' @noRd
req_error <- function(...) httr2::req_error(...)

#' @noRd
req_options <- function(...) httr2::req_options(...)

#' @noRd
req_perform <- function(...) httr2::req_perform(...)

#' @noRd
resp_status <- function(...) httr2::resp_status(...)

#' @noRd
resp_body_raw <- function(...) httr2::resp_body_raw(...)

#' @noRd
write_file_raw <- function(...) brio::write_file_raw(...)

fake_req <- structure(list(), class = "httr2_request")
fake_resp <- structure(list(), class = "httr2_response")

mock_names <- c(
  "fs_dir_create",
  "fs_path_dir",
  "fs_path_temp",
  "req_request",
  "req_user_agent",
  "req_headers",
  "req_retry",
  "req_cache",
  "req_error",
  "req_options",
  "req_perform",
  "resp_status",
  "resp_body_raw",
  "write_file_raw"
)

default_mocks <- list(
  fs_dir_create = function(...) invisible(NULL),
  fs_path_dir = function(path, ...) "/tmp",
  fs_path_temp = function(...) "/tmp",

  req_request = function(...) fake_req,
  req_user_agent = function(req, ...) req,
  req_headers = function(req, ...) req,
  req_retry = function(req, max_tries, ...) req,
  req_cache = function(req, ...) req,
  req_error = function(req, ...) req,
  req_options = function(req, ...) req,
  req_perform = function(req, ...) fake_resp,

  resp_status = function(...) 200L,
  resp_body_raw = function(...) as.raw(c(0x68, 0x69)),

  write_file_raw = function(...) invisible(NULL)
)

merge_mocks <- function(defaults, overrides = list()) {
  if (length(overrides) == 0) {
    return(defaults)
  }

  defaults[names(overrides)] <- overrides
  defaults
}

local_retry_download_mocks <- function(overrides = list()) {
  local_mocked_bindings(
    .env = getNamespace("getCRUCLdata"),
    !!!merge_mocks(default_mocks[mock_names], overrides)
  )
}

test_that(".retry_download returns dest invisibly on success", {
  dest <- "/tmp/file.nc"
  local_retry_download_mocks()

  result <- withVisible(.retry_download(
    "https://example.com/file.nc",
    dest
  ))

  expect_identical(result$value, dest)
  expect_false(result$visible)
})

test_that(".retry_download aborts on HTTP 404", {
  local_retry_download_mocks(
    list(resp_status = function(...) 404L)
  )

  expect_error(
    .retry_download("https://example.com/missing.nc", "/tmp/missing.nc"),
    regexp = "HTTP 404"
  )
})

test_that(".retry_download aborts on HTTP 500", {
  local_retry_download_mocks(
    list(resp_status = function(...) 500L)
  )

  expect_error(
    .retry_download("https://example.com/broken.nc", "/tmp/broken.nc"),
    regexp = "HTTP 500"
  )
})

test_that(".retry_download does not abort below the 400 threshold", {
  local_retry_download_mocks(
    list(resp_status = function(...) 399L)
  )

  expect_no_error(
    .retry_download("https://example.com/ok.nc", "/tmp/ok.nc")
  )
})

test_that(".retry_download writes the raw response body to dest", {
  raw_bytes <- as.raw(c(0x01, 0x02, 0x03))
  written_path <- NULL
  written_bytes <- NULL

  local_retry_download_mocks(
    list(
      resp_body_raw = function(...) raw_bytes,
      write_file_raw = function(data, path) {
        written_bytes <<- data
        written_path <<- path
      }
    )
  )

  .retry_download("https://example.com/data.nc", "/tmp/data.nc")

  expect_equal(written_bytes, raw_bytes)
  expect_equal(written_path, "/tmp/data.nc")
})

test_that(".retry_download creates the destination directory", {
  created_dir <- NULL

  local_retry_download_mocks(
    list(
      fs_path_dir = function(path, ...) "/some/nested/dir",
      fs_dir_create = function(path, ...) {
        created_dir <<- path
      }
    )
  )

  .retry_download("https://example.com/data.nc", "/some/nested/dir/data.nc")

  expect_identical(created_dir, "/some/nested/dir")
})

test_that(".retry_download forwards .max_tries to req_retry", {
  captured_max_tries <- NULL

  local_retry_download_mocks(
    list(
      req_retry = function(req, max_tries, ...) {
        captured_max_tries <<- max_tries
        req
      }
    )
  )

  .retry_download(
    "https://example.com/data.nc",
    "/tmp/data.nc",
    .max_tries = 7L
  )

  expect_identical(captured_max_tries, 7L)
})

test_that(".retry_download error message includes the URL and status code", {
  url <- "https://example.com/secret.nc"

  local_retry_download_mocks(
    list(resp_status = function(...) 403L)
  )

  expect_error(
    .retry_download(url, "/tmp/secret.nc"),
    regexp = "secret\\.nc.*403|403.*secret\\.nc"
  )
})

.bad_coords <- data.table::data.table(
  lat = c(30.917, 31.083, 31.250, 31.417, 31.750, 31.917),
  lon = c(35.417, 35.417, 35.417, 35.417, 35.583, 35.583)
)

.cru_month_names <- c(
  "jan",
  "feb",
  "mar",
  "apr",
  "may",
  "jun",
  "jul",
  "aug",
  "sep",
  "oct",
  "nov",
  "dec"
)

.cru_files <- c(
  pre = "grid_10min_pre.dat.gz",
  rd0 = "grid_10min_rd0.dat.gz",
  tmp = "grid_10min_tmp.dat.gz",
  dtr = "grid_10min_dtr.dat.gz",
  reh = "grid_10min_reh.dat.gz",
  sunp = "grid_10min_sunp.dat.gz",
  frs = "grid_10min_frs.dat.gz",
  wnd = "grid_10min_wnd.dat.gz",
  elv = "grid_10min_elv.dat.gz"
)

# Mock .retry_download() so tests never hit the network
mock_retry_download <- function(url, dest, .max_tries = 3L) {
  fname <- fs::path_file(url)
  src <- fixture_path(fname)

  if (!fs::file_exists(src)) {
    stop("Mock download: fixture not found: ", fname, call. = FALSE)
  }

  fs::file_copy(src, dest, overwrite = TRUE)
  invisible(dest)
}

#' Create a data.table of CRU CL v. 2.0 climatology elements
#'
#' Automates importing \acronym{CRU} \acronym{CL} v.2.0 climatology data and
#' from either the CRU server or local files and creates a \CRANpkg{data.table}
#' of the data. If requested, minimum and maximum temperature may also be
#' calculated as described in the data
#' [readme.txt](https://crudata.uea.ac.uk/cru/data/hrg/tmc/readme.txt) file and
#' returned.
#'
#' @section Nomenclature and Units:
#' \describe{
#' \item{pre}{precipitation (millimetres/month)}
#'   \describe{
#'    \item{cv}{cv of precipitation (percent)}
#'   }
#' \item{rd0}{wet-days (number days with >0.1 millimetres rain per month)}
#' \item{tmp}{mean temperature (degrees Celsius)}
#' \item{dtr}{mean diurnal temperature range (degrees Celsius)}
#' \item{reh}{relative humidity (percent)}
#' \item{sunp}{sunshine (percent of maximum possible (percent of day length))}
#' \item{frs}{ground-frost (number of days with ground-frost per month)}
#' \item{wnd}{10 metre wind speed (metres/second)}
#' \item{elv}{elevation (automatically converted to metres from kilometres)}
#' }
#' For more information see the description of the data provided by
#' \acronym{CRU}, <https://crudata.uea.ac.uk/cru/data/hrg/tmc/readme.txt>
#'
#' @param pre Read precipitation (millimetres/month) and return it, `TRUE`.
#'  Defaults to `FALSE`.
#' @param pre_cv Read cv of precipitation (percent) and return it, `TRUE`.
#'  Defaults to `FALSE`. NOTE. Setting this to `TRUE` will always results in
#'  `pre` being set to `TRUE` and returned as well.
#' @param rd0 Read wet-days (number days with >0.1 millimetres rain per
#'  month) and return it, `TRUE`. Defaults to `FALSE`.
#' @param dtr Read mean diurnal temperature range (degrees Celsius)
#'  and return it, `TRUE`. Defaults to `FALSE`.
#' @param tmp Reads temperature (degrees Celsius) and return it, TRUE`.
#'  Defaults to `FALSE`.
#' @param tmn Calculate minimum temperature values (degrees Celsius)
#'  and return it, `TRUE`. Defaults to `FALSE`.
#' @param tmx Calculate maximum temperature (degrees Celsius) and return it,
#'  `TRUE`. Defaults to `FALSE`.
#' @param reh Read relative humidity and return it, `TRUE`. Defaults to
#'  `FALSE`.
#' @param sunp Read sunshine, percent of maximum possible (percent of
#'  day length) and return it, `TRUE`. Defaults to `FALSE`.
#' @param frs Read ground-frost records (number of days with ground-
#'  frost per month) and return it, `TRUE`. Defaults to `FALSE`.
#' @param wnd Read 10 m wind speed (metres/second) and return it, `TRUE`.
#'  Defaults to `FALSE`.
#' @param elv Read elevation (converted to metres) and return it, `TRUE`.
#'  Defaults to `FALSE`.
#' @param x An optional local file path where \acronym{CRU} \acronym{CL} v.2.0
#'  .dat.gz files are located.  If this is empty, the requested data will
#'  automatically be downloaded from the server.
#'
#' @examplesIf interactive()
#' # Create a data frame of temperature from locally available files in the
#' # tempdir() directory.
#' library(fs)
#'
#' f <- path(path_temp(), "grid_10min_tmp.dat.gz")
#'
#' download.file(
#'   url = "https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_tmp.dat.gz",
#'   destfile = f
#' )
#'
#'
#' cru_tmp <- read_cru_dt(tmp = TRUE, x = f)
#'
#' cru_tmp
#'
#' # or downloading directly from the CRU server
#'
#' cru_tmp <- read_cru_dt(tmp = TRUE)
#'
#' cru_tmp
#'
#' @seealso [read_cru_rast].
#'
#' @returns A [data.table::data.table] object of \acronym{CRU} \acronym{CL} v.
#'  2.0 climatology elements.
#'
#' @author Adam H. Sparks, \email{adamhsparks@@gmail.com}
#'
#' @source
#' \describe{
#'  \item{pre}{<https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_pre.dat.gz>}
#'  \item{rd0}{<https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_rd0.dat.gz>}
#'  \item{tmp}{<https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_tmp.dat.gz>}
#'  \item{dtr}{<https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_dtr.dat.gz>}
#'  \item{reh}{<https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_reh.dat.gz>}
#'  \item{sunp}{<https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_sunp.dat.gz>}
#'  \item{frs}{<https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_frs.dat.gz>}
#'  \item{wnd}{<https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_wnd.dat.gz>, areas originally including Antarctica are removed.}
#'  \item{elv}{<https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_elv.dat.gz>, values are converted from kilometres to metres.}
#' }
#' This package crops all spatial outputs to an extent of ymin = -60, ymax = 85,
#' xmin = -180, xmax = 180.
#'
#' @references New, Mark, et al. "A high-resolution data set of surface climate
#'  over global land areas." Climate research 21.1 (2002): 1-25.
#'  <https://crudata.uea.ac.uk/cru/data/hrg/tmc/new_et_al_10minute_climate_CR.pdf>
#'
#' @export
read_cru_dt <- function(
  pre = FALSE,
  pre_cv = FALSE,
  rd0 = FALSE,
  tmp = FALSE,
  dtr = FALSE,
  reh = FALSE,
  tmn = FALSE,
  tmx = FALSE,
  sunp = FALSE,
  frs = FALSE,
  wnd = FALSE,
  elv = FALSE,
  x = NULL
) {
  vars <- c(
    pre = pre,
    pre_cv = pre_cv,
    rd0 = rd0,
    tmp = tmp,
    dtr = dtr,
    reh = reh,
    tmn = tmn,
    tmx = tmx,
    sunp = sunp,
    frs = frs,
    wnd = wnd,
    elv = elv
  )

  files <- .validate_filter_files(vars, x)
  resolved <- .file_handling(vars, files, x)

  .create_dt(resolved$vars, resolved$files)
}

test_that("read_cru_dt loads CRU data from fixtures", {
  local_d <- withr::local_tempdir()
  fs::dir_create(local_d) # <-- FORCE creation of directory

  expect_true(fs::is_dir(local_d))

  fs::file_copy(
    fs::dir_ls(test_path("fixtures"), glob = "*.dat.gz"),
    local_d,
    overwrite = TRUE
  )

  df <- read_cru_dt(
    pre = TRUE,
    pre_cv = TRUE,
    tmp = TRUE,
    x = local_d
  )

  expect_s3_class(df, "data.table")
  expect_true(all(
    c("lat", "lon", "month", "pre", "pre_cv", "tmp") %in% names(df)
  ))
})

test_that("read_cru_dt uses mocked download instead of network", {
  with_mocked_bindings(
    .retry_download = mock_retry_download,
    .package = "getCRUCLdata",
    {
      df <- read_cru_dt(pre = TRUE, tmp = TRUE, x = NULL)
      expect_s3_class(df, "data.table")
      expect_true(all(c("pre", "tmp") %in% names(df)))
    }
  )
})

#' Resolve CRU file handling for local or remote sources
#'
#' @param vars Named logical vector.
#' @param files Character vector of CRU file names.
#' @param x Optional path.
#'
#' @returns List with vars and files.
#' @dev

.file_handling <- function(vars, files, x) {
  vars <- .normalize_vars(vars)

  # --- Case 1: remote download ---
  if (is.null(x)) {
    base <- "https://crudata.uea.ac.uk/cru/data/hrg/tmc"
    urls <- fs::path(base, files)
    dest <- fs::path_temp(fs::path_file(files))

    # side effects only
    invisible(Map(.retry_download, urls, dest))

    return(list(vars = vars, files = dest))
  }

  # --- Local cases ---
  x <- fs::path_abs(fs::path_norm(trimws(x)))

  # Case 2: directory
  if (fs::is_dir(x)) {
    expected <- fs::path(x, files)
    existing <- expected[fs::file_exists(expected)]

    if (!length(existing)) {
      cli::cli_abort(
        "No CRU CL 2.0 data files were found in {.var x} = {x}.",
        call = rlang::caller_env()
      )
    }

    return(list(vars = vars, files = existing))
  }

  # Case 3: single .gz file
  if (identical(fs::path_ext(x), "gz")) {
    if (!fs::file_exists(x)) {
      cli::cli_abort(
        "The file {.var x} does not exist: {x}.",
        call = rlang::caller_env()
      )
    }

    return(list(vars = vars, files = x))
  }

  # --- Invalid input ---
  cli::cli_abort(
    "{.var x} must be either a directory or a .gz file.",
    call = rlang::caller_env()
  )
}

# Generated by roxyglobals: do not edit by hand

utils::globalVariables(c(
  "month", # <.create_dt>
  "lat", # <.create_dt>
  "lon", # <.create_dt>
  "tmx", # <.add_derived_dt>
  "tmp", # <.add_derived_dt>
  "dtr", # <.add_derived_dt>
  "tmn", # <.add_derived_dt>
  "tmp", # <.drop_source_vars_dt>
  "dtr", # <.drop_source_vars_dt>
  "month", # <.dt_to_rast>
  "lon", # <.remove_bad_cells_rast>
  "lat", # <.remove_bad_cells_rast>
  "lat", # <.read_local_files>
  "lon", # <.read_local_files>
  "month", # <.read_local_files>
  "pre_cv", # <.read_local_files>
  "i.pre_cv", # <.read_local_files>
  "elv", # <.read_local_files>
  NULL
))

#' Create a data.table from CRU data
#'
#' @param vars Named logical vector.
#' @param files Character vector of CRU file paths.
#'
#' @returns A data.table.
#' @autoglobal
#' @dev
.create_dt <- function(vars, files) {
  tidy_dt <- if (is.character(files)) .tidy_dt(vars, files) else files
  tidy_dt <- .add_derived_dt(tidy_dt, vars)
  tidy_dt <- .drop_source_vars_dt(tidy_dt, vars)
  tidy_dt[, month := factor(month, levels = .cru_month_names, ordered = TRUE)]
  data.table::setorder(tidy_dt, month)
  tidy_dt[, `:=`(
    lat = round(lat, 3),
    lon = round(lon, 3)
  )]
  if (isTRUE(vars["elv"])) {
    tidy_dt <- tidy_dt[!.bad_coords, on = list(lat, lon)]
  }
  # Crop to CRU CL 2.0 extent: ymin = -60, ymax = 85
  tidy_dt <- tidy_dt[lat >= -60 & lat <= 85]
  return(tidy_dt[])
}

#' Add derived temperature values for TMAX and TMIN
#' @param dt A data.table object of CRU data containing `tmp` and `dtr` in order
#'  to calculate tmin and tmax.
#' @param vars A named logical vector.
#' @returns A data.table with tmin and or tmax cols.
#' @autoglobal
.add_derived_dt <- function(dt, vars) {
  has_inputs <- all(c("tmp", "dtr") %in% names(dt))
  if (has_inputs) {
    if (isTRUE(vars["tmx"])) {
      dt[, tmx := tmp + 0.5 * dtr]
    }
    if (isTRUE(vars["tmn"])) {
      dt[, tmn := tmp - 0.5 * dtr]
    }
  }
  return(dt)
}


#' Remove source vars if not requested
#'
#' If `tmin` or `tmax` are requested but `tmp` or `dtr` are not requested, this
#'  function drops the unrequested vars.
#' @param dt A data.table object of CRU data containing `tmp` and `dtr` in order
#'  to calculate tmin and tmax.
#' @param vars A named logical vector.
#' @returns A data.table with the unrequested var columns removed.
#' @autoglobal
.drop_source_vars_dt <- function(dt, vars) {
  if (isTRUE(vars["tmn"]) || isTRUE(vars["tmx"])) {
    if (!vars["tmp"]) {
      dt[, tmp := NULL]
    }
    if (!vars["dtr"]) dt[, dtr := NULL]
  }
  return(dt)
}

#' Validate user-supplied file or directory path
#'
#' @param x Path.
#' @param files Expected CRU files.
#'
#' @returns Absolute validated path.
#' @dev

.validate_x <- function(x, files) {
  x <- fs::path_abs(fs::path_norm(trimws(x)))

  # Case 1: directory
  if (fs::is_dir(x)) {
    return(x)
  }

  # Case 2: must be a single .gz file
  if (fs::path_ext(x) != "gz") {
    cli::cli_abort(
      "{.var x} must be either a directory or a .gz file.",
      call = rlang::caller_env()
    )
  }

  dir_path <- fs::path_dir(x)

  # Directory must exist
  if (!fs::dir_exists(dir_path)) {
    cli::cli_abort(
      "The directory for {.var x} does not exist: {dir_path}.",
      call = rlang::caller_env()
    )
  }

  # If multiple variables requested → treat x as directory
  if (length(files) > 1L) {
    cli::cli_warn(
      c(
        "You supplied a single file {.var x} but requested multiple variables.",
        "Interpreting the path at the directory level instead.",
        "All matching files in {dir_path} will be considered."
      ),
      call = rlang::caller_env()
    )
    return(dir_path)
  }

  # Valid single .gz file
  return(x)
}

test_that(".validate_x() returns absolute path for valid directory", {
  # Create a temporary directory
  temp_dir <- tempdir()

  result <- .validate_x(temp_dir, "tmp")

  expect_true(fs::is_absolute_path(result))
  expect_true(fs::is_dir(result))
})

test_that(".validate_x() normalizes and trims path input", {
  # Create a temporary directory
  temp_dir <- tempdir()

  # Test with extra whitespace and path normalization
  messy_path <- paste0("  ", temp_dir, "  ")

  result <- .validate_x(messy_path, "tmp")

  expect_identical(result, fs::path_abs(fs::path_norm(temp_dir)))
})

test_that(".validate_x() warns and returns directory when multiple files requested with single .gz", {
  temp_dir <- tempdir()
  temp_file <- fs::path(temp_dir, "test.gz")
  fs::file_create(temp_file)

  # Convert to plain character string and normalize
  temp_dir_normalized <- as.character(fs::path_abs(fs::path_norm(temp_dir)))

  # Suppress warning for this test since we're checking the result
  result <- suppressWarnings(
    .validate_x(temp_file, c("tmp", "pre"))
  )
  expect_identical(result, temp_dir_normalized)

  # Separately verify the warning is raised
  expect_warning(
    .validate_x(temp_file, c("tmp", "pre")),
    "You supplied a single file"
  )

  # Cleanup
  fs::file_delete(temp_file)
})

test_that(".validate_x() throws error when .gz file directory does not exist", {
  nonexistent_dir <- "/nonexistent/path/that/does/not/exist"
  nonexistent_file <- fs::path(nonexistent_dir, "test.gz")

  expect_error(.validate_x(nonexistent_file, "tmp"))
})

test_that(".validate_x() returns .gz file when single variable requested", {
  temp_dir <- tempdir()
  temp_file <- fs::path(temp_dir, "test.gz")
  fs::file_create(temp_file)

  result <- .validate_x(temp_file, "tmp")

  expect_equal(result, fs::path_abs(fs::path_norm(temp_file)))

  # Cleanup
  fs::file_delete(temp_file)
})

test_that(".validate_x() handles relative paths and converts to absolute", {
  # Create a temporary directory
  temp_dir <- tempdir()
  original_wd <- getwd()
  tryCatch(
    {
      setwd(temp_dir)

      # Create a test subdirectory
      subdir <- "test_subdir"
      fs::dir_create(subdir)

      # Use relative path
      result <- .validate_x(subdir, "tmp")

      expect_true(fs::is_absolute_path(result))
      expect_true(fs::is_dir(result))

      # Cleanup
      fs::dir_delete(subdir)
    },
    finally = {
      setwd(original_wd)
    }
  )
})

test_that(".validate_x() handles paths with trailing slashes", {
  temp_dir <- tempdir()
  path_with_slash <- paste0(temp_dir, .Platform$file.sep)

  result <- .validate_x(path_with_slash, "tmp")

  expect_true(fs::is_dir(result))
})

test_that(".validate_x() correctly identifies .gz extension", {
  temp_dir <- tempdir()

  # Test with .gz file
  gz_file <- fs::path(temp_dir, "data.gz")
  fs::file_create(gz_file)

  result_gz <- .validate_x(gz_file, "tmp")
  expect_equal(fs::path_ext(result_gz), "gz")

  # Test with .tar.gz file (should still be treated as .gz)
  tar_gz_file <- fs::path(temp_dir, "data.tar.gz")
  fs::file_create(tar_gz_file)

  result_tar_gz <- .validate_x(tar_gz_file, "tmp")
  expect_equal(fs::path_ext(result_tar_gz), "gz")

  # Cleanup
  fs::file_delete(c(gz_file, tar_gz_file))
})

test_that(".validate_x() warns when multiple files with single .gz file", {
  temp_dir <- tempdir()
  temp_file <- fs::path(temp_dir, "test.gz")
  fs::file_create(temp_file)

  # Check that warning message contains expected text
  expect_warning(
    .validate_x(temp_file, c("tmp", "pre", "cld")),
    "multiple variables"
  )

  # Cleanup
  fs::file_delete(temp_file)
})

test_that(".validate_x() handles special characters in path", {
  temp_dir <- tempdir()
  # Create a subdirectory with spaces in name
  special_dir <- fs::path(temp_dir, "dir with spaces")
  fs::dir_create(special_dir)

  result <- .validate_x(special_dir, "tmp")

  expect_true(fs::is_dir(result))

  # Cleanup
  fs::dir_delete(special_dir)
})

test_that(".validate_x() throws error for non-existent directory path", {
  nonexistent_dir <- "/this/path/does/not/exist/at/all"

  expect_error(.validate_x(nonexistent_dir, "tmp"))
})

test_that(".validate_x() returns directory without error when valid directory", {
  temp_dir <- tempdir()

  # Should not error
  expect_no_error(
    result <- .validate_x(temp_dir, "tmp")
  )

  expect_equal(result, fs::path_abs(fs::path_norm(temp_dir)))
})

test_that(".validate_x() handles empty files parameter", {
  temp_dir <- tempdir()

  result <- .validate_x(temp_dir, character(0))

  expect_true(fs::is_dir(result))
})

test_that(".validate_x() returns directory with single file when no variables", {
  temp_dir <- tempdir()
  temp_file <- fs::path(temp_dir, "test.gz")
  fs::file_create(temp_file)

  # Empty files vector should behave like single file request
  result <- .validate_x(temp_file, character(0))

  # Should return the file path (no warning since length(files) is 0, not > 1)
  expect_equal(result, fs::path_abs(fs::path_norm(temp_file)))

  # Cleanup
  fs::file_delete(temp_file)
})

test_that(".validate_x() correctly determines files length for warning condition", {
  temp_dir <- tempdir()
  temp_file <- fs::path(temp_dir, "test.gz")
  fs::file_create(temp_file)

  # Exactly 2 files should trigger warning
  expect_warning(
    .validate_x(temp_file, c("tmp", "pre"))
  )

  # Exactly 1 file should not warn
  expect_no_warning(
    .validate_x(temp_file, "tmp")
  )

  # Cleanup
  fs::file_delete(temp_file)
})

test_that(".validate_x() preserves .gz extension in returned path", {
  temp_dir <- tempdir()
  gz_file <- fs::path(temp_dir, "cru_tmp_2020.gz")
  fs::file_create(gz_file)

  result <- .validate_x(gz_file, "tmp")

  expect_true(grepl("\\.gz$", result))

  # Cleanup
  fs::file_delete(gz_file)
})

#' Check that at least one variable is requested
#'
#' @param vars Named logical vector of CRU variable selections.
#'
#' @returns Invisible NULL.
#' @dev

.check_vars <- function(vars) {
  if (is.null(vars) || !is.logical(vars)) {
    cli::cli_abort("Internal error: {.arg vars} must be a logical vector.")
  }

  if (!any(vars)) {
    cli::cli_abort(
      "You must select at least one element for download or import."
    )
  }

  invisible(TRUE)
}

#' Normalize dependent CRU variable selections
#'
#' @param vars Named logical vector.
#'
#' @returns Normalized logical vector.
#' @dev

.normalize_vars <- function(vars) {
  # pre_cv implies pre
  vars["pre"] <- isTRUE(vars["pre"]) || isTRUE(vars["pre_cv"])

  # tmn/tmx imply tmp + dtr
  need_temp <- isTRUE(vars["tmn"]) || isTRUE(vars["tmx"])
  vars["tmp"] <- isTRUE(vars["tmp"]) || need_temp
  vars["dtr"] <- isTRUE(vars["dtr"]) || need_temp

  return(vars)
}

#' Internal helper for creating terra layers
#'
#' @param file CRU .dat.gz file path.
#' @param wrld Empty `terra::rast` template.
#' @param vars Named logical vector of CRU variable selections.
#'
#' @returns A [terra::rast] object for one variable.
#' @dev

.make_rast <- function(file, wrld, vars, varname) {
  wvar <- data.table::fread(file, header = FALSE)
  cells <- terra::cellFromXY(wrld, wvar[, c(2L, 1L)])
  n <- ncol(wvar)

  # Helper: build a list of 12 monthly layers from column indices
  build_monthly <- function(cols, prefix = NULL) {
    layers <- lapply(seq_along(cols), function(i) {
      r <- terra::rast(wrld)
      r[cells] <- wvar[[cols[i]]]
      r
    })
    names(layers) <- if (is.null(prefix)) {
      .cru_month_names
    } else {
      sprintf("%s%s", prefix, .cru_month_names)
    }
    layers
  }

  # Helper: build a single-layer raster
  build_single <- function(col, name) {
    r <- terra::rast(wrld)
    r[cells] <- wvar[[col]]
    names(r) <- name
    r
  }

  # --- Case 1: Standard 12-month variables (14 columns) ---
  if (n == 14L) {
    layers <- build_monthly(3:14)
    names(layers) <- sprintf("%s_%s", varname, .cru_month_names)
    return(terra::rast(layers))
  }

  # --- Case 2: pre + pre_cv (26 columns) ---
  if (n == 26L) {
    pre_layers <- build_monthly(3:14, "pre_")
    if (!isTRUE(vars["pre_cv"])) {
      return(terra::rast(pre_layers))
    }
    cv_layers <- build_monthly(15:26, "pre_cv_")
    return(terra::rast(c(pre_layers, cv_layers)))
  }

  # --- Case 3: elevation (3 columns) ---
  if (n == 3L) {
    r <- build_single(3L, "elv")
    r <- r * 1000L # km -> m first
    r <- .remove_bad_cells_rast(r) # then mask
    return(r)
  }

  cli::cli_abort("Unexpected file format in {.path {file}}.")
}

---
title: 'getCRUCLdata: Use and Explore CRU CL v. 2.0 Climatology Elements in R'
authors:
- affiliation: 1
  name: Adam H Sparks
  orcid: 0000-0002-0061-8359
output: pdf_document
tags:
- climate
- R
- applied climatology
- high resolution surface
- data
affiliations:
  index: 1
  name: University of Southern Queensland, Centre for Crop Health, Toowoomba Queensland 4350, Australia
bibliography: paper.bib
date: "04 April 2017"

---

# Summary

The CRU CL v. 2.0 data are a gridded climatology of 1961-1990 monthly means released in 2002 and cover all land areas (excluding Antarctica) at 10 arcminutes (0.1666667 degree) resolution [@New2002] providing precipitation, cv of precipitation, wet-days, mean temperature, mean diurnal temperature range, relative humidity, sunshine, ground-frost, windspeed and elevation. While these data have a high resolution and are freely available, the data format can be cumbersome for working with. Four functions are provided by _getCRUCLdata_ that automate importing these data into R [@R-base]. All of the functions facilitate the calculation of minimum temperature and maximum temperature, and format the data into a tidy data frame [@Wickham2014] in a _tibble_ [@Wickham2017] object or a list of _raster_ stack objects [@Raster] for use in R or easily exported to a raster format file for use in a geographic information system (GIS). Two functions, `get_CRU_df()` and `get_CRU_stack()` provide the ability to easily download CRU CL v. 2.0 data from the CRU website and import the data into R and allow for caching downloaded data. The other two functions, `create_CRU_df()` and `create_CRU_stack()` allow the user to easily import the data files from a local disk location and transform them into a tidy data frame _tibble_ or _raster_ stack. The data have applications in applied climatology, biogeochemical modelling, hydrology and agricultural meteorology [@New2002].

# References

---
title: "getCRUCLdata"
author: "Adam H. Sparks"
date: "2026-05-16"
output:
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteEngine{knitr::rmarkdown_notangle}
  %\VignetteIndexEntry{getCRUCLdata}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{data.table}
  %\VignetteDepends{terra}
---



## Introduction to {getCRUCLdata}

The {getCRUCLdata} package provides functions that automate importing CRU CL v. 2.0 climatology data into R, facilitate the calculation of minimum temperature and maximum temperature, and formats the data into a [data.table::data.table] object or a [terra:SpatRaster].

CRU CL v. 2.0 data are a gridded climatology of 1961-1990 monthly means released in 2002 and cover all land areas (excluding Antarctica) at 10 arcminutes (0.1666667 arc degree) resolution.
For more information see the description of the data provided by the University of East Anglia Climate Research Unit (CRU), <https://crudata.uea.ac.uk/cru/data/hrg/tmc/readme.txt>.

## Changes to original CRU CL v. 2.0 data

This package automatically converts elevation values from kilometres to metres.

This package crops all spatial outputs to an extent of ymin = -60, ymax = 85, xmin = -180, xmax = 180.
Note that the original wind data include land area for parts of Antarctica, and so these are omitted.
There are also some elevation values around the Dead Sea area that do not have corresponding climate values and so these are removed.
What is left is a complete suite of data for climate and elevation data in all grid cells that are provided, but not *all* the data that are in the original datasets.

# Using {getCRUCLdata}

Logical arguments are used to specify the climatology elements to retrieve and parse.
All data arguments default to `FALSE`.
The arguments for selecting the climatology elements for importing are:

- **pre** Logical. Read precipitation (millimetres/month) from server and return in the data?

- **pre_cv** Logical. Read cv of precipitation (percent) from server and return in the data?

- **rd0** Logical. Read wet-days (number days with >0.1 millimetres rain per month) and return in the data?

- **dtr** Logical. Read mean diurnal temperature range (degrees Celsius) and return it in the data?

- **tmp** Logical. Read temperature (degrees Celsius) and return it in the data?

- **tmn** Logical. Calculate minimum temperature values (degrees Celsius) and return it in the data?

- **tmx** Logical. Calculate maximum temperature (degrees Celsius) and return it in the data?

- **reh** Logical. Read relative humidity and return it in the data?

- **sunp** Logical. Read sunshine, percent of maximum possible (percent of day length) and return it in data?

- **frs** Logical. Read ground-frost records (number of days with ground-frost per month) and return it in data?

- **wnd** Logical. Read 10m wind speed (metres/second) and return it in the data?

- **elv** Logical. Read elevation (and convert to metres from kilometres) and return it in the data?

- **x** String. Local file path where CRU CL v. 2.0 .dat.gz files are located.

### Creating data frames for use in R

The `read_cru_dt()` function automates the download process and creates data.tables of the climatology elements.


``` r
library(getCRUCLdata)
#> 
#> Attaching package: 'getCRUCLdata'
#> The following object is masked _by_ '.GlobalEnv':
#> 
#>     read_cru_dt

CRU_data <- read_cru_dt(
  pre = TRUE,
  pre_cv = TRUE,
  rd0 = TRUE,
  tmp = TRUE,
  dtr = TRUE,
  reh = TRUE,
  tmn = TRUE,
  tmx = TRUE,
  sunp = TRUE,
  frs = TRUE,
  wnd = TRUE,
  elv = TRUE
)
#> Error in `.validate_filter_files()`:
#> ! could not find function ".validate_filter_files"

CRU_data
#> Error:
#> ! object 'CRU_data' not found
```

Perhaps you only need one or two elements, it is easy to create a data.table of mean temperature only.


``` r
t <- read_cru_dt(tmp = TRUE)
#> Error in `.validate_filter_files()`:
#> ! could not find function ".validate_filter_files"

t
#> function (x) 
#> UseMethod("t")
#> <bytecode: 0x75f380708>
#> <environment: namespace:base>
```

### Creating terra SpatRaster objects

For working with spatial data, {getCRUCLdata} provides a function that create a [terra::SpatRaster] object of the data.

The `read_cru_rast()` functions provide similar functionality to `read_cru_dt()`, but rather than returning a data frame, it returns a `terra::SpatRaster()` object with n layers for use in an R session.
Illustrated here is creating a `terra::SpatRaster()` of all CRU CL v. 2.0 climatology elements available.


``` r
CRU_rast <- read_cru_rast(
  pre = TRUE,
  pre_cv = TRUE,
  rd0 = TRUE,
  tmp = TRUE,
  dtr = TRUE,
  reh = TRUE,
  tmn = TRUE,
  tmx = TRUE,
  sunp = TRUE,
  frs = TRUE,
  wnd = TRUE,
  elv = TRUE
)

CRU_rast
#> class       : SpatRaster
#> size        : 870, 2160, 133  (nrow, ncol, nlyr)
#> resolution  : 0.1666667, 0.1666667  (x, y)
#> extent      : -180, 180, -60, 85  (xmin, xmax, ymin, ymax)
#> coord. ref. : lon/lat WGS 84 (CRS84) (OGC:CRS84)
#> source(s)   : memory
#> names       : pre_1, pre_2, pre_3, pre_4, pre_5,  pre_6, ...
#> min values  :     0,     0,     0,     0,     0,      0, ...
#> max values  : 910.1, 824.3, 727.3, 741.3,  1100, 2512.6, ...
```

## Reading files downloaded outside R

Both `read_cru_dt()` and `read_cru_rast` provide capability for users that may have connectivity issues or simply wish to use something other than R to download the data files.
You may supply the location of the files on the local disk, `x`, that you wish to import.


``` r
t <- create_CRU_df(tmp = TRUE, x = "~/Downloads")
```



# CRU CL v. 2.0 reference and abstract

Mark New (1,*), David Lister (2), Mike Hulme (3), Ian Makin (4)
A high-resolution data set of surface climate over global land areas Climate Research, 2000, Vol 21, pg 1-25
(1) School of Geography and the Environment, University of Oxford,
    Mansfield Road, Oxford OX1 3TB, United Kingdom
(2) Climatic Research Unit, and (3) Tyndall Centre for Climate Change Research,
    both at School of Environmental Sciences, University of East Anglia,
    Norwich NR4 7TJ, United Kingdom
(4) International Water Management Institute, PO Box 2075, Colombo, Sri Lanka

**ABSTRACT:** We describe the construction of a 10-minute latitude/longitude
data set of mean monthly surface climate over global land areas, excluding
Antarctica. The climatology includes 8 climate elements - precipitation, wet-day
frequency, temperature, diurnal temperature range, relative humidity, sunshine
duration, ground frost frequency and windspeed - and was interpolated from a
data set of station means for the period centred on 1961 to 1990. Precipitation
was first defined in terms of the parameters of the Gamma distribution, enabling
the calculation of monthly precipitation at any given return period. The data
are compared to an earlier data set at 0.5 degrees latitude/longitude resolution
and show added value over most regions. The data will have many applications in
applied climatology, biogeochemical modelling, hydrology and agricultural
meteorology and are available through the School of Geography Oxford
(<https://www.geog.ox.ac.uk>), the International Water Management Institute
"World Water and Climate Atlas" (<https://www.iwmi.org/>) and the Climatic
Research Unit (<https://www.uea.ac.uk/groups-and-centres/climatic-research-unit>).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_cru_dt.R
\name{read_cru_dt}
\alias{read_cru_dt}
\title{Create a data.table of CRU CL v. 2.0 climatology elements}
\source{
\describe{
\item{pre}{\url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_pre.dat.gz}}
\item{rd0}{\url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_rd0.dat.gz}}
\item{tmp}{\url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_tmp.dat.gz}}
\item{dtr}{\url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_dtr.dat.gz}}
\item{reh}{\url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_reh.dat.gz}}
\item{sunp}{\url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_sunp.dat.gz}}
\item{frs}{\url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_frs.dat.gz}}
\item{wnd}{\url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_wnd.dat.gz}, areas originally including Antarctica are removed.}
\item{elv}{\url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_elv.dat.gz}, values are converted from kilometres to metres.}
}
This package crops all spatial outputs to an extent of ymin = -60, ymax = 85,
xmin = -180, xmax = 180.
}
\usage{
read_cru_dt(
  pre = FALSE,
  pre_cv = FALSE,
  rd0 = FALSE,
  tmp = FALSE,
  dtr = FALSE,
  reh = FALSE,
  tmn = FALSE,
  tmx = FALSE,
  sunp = FALSE,
  frs = FALSE,
  wnd = FALSE,
  elv = FALSE,
  x = NULL
)
}
\arguments{
\item{pre}{Read precipitation (millimetres/month) and return it, \code{TRUE}.
Defaults to \code{FALSE}.}

\item{pre_cv}{Read cv of precipitation (percent) and return it, \code{TRUE}.
Defaults to \code{FALSE}. NOTE. Setting this to \code{TRUE} will always results in
\code{pre} being set to \code{TRUE} and returned as well.}

\item{rd0}{Read wet-days (number days with >0.1 millimetres rain per
month) and return it, \code{TRUE}. Defaults to \code{FALSE}.}

\item{tmp}{Reads temperature (degrees Celsius) and return it, TRUE\verb{. Defaults to }FALSE`.}

\item{dtr}{Read mean diurnal temperature range (degrees Celsius)
and return it, \code{TRUE}. Defaults to \code{FALSE}.}

\item{reh}{Read relative humidity and return it, \code{TRUE}. Defaults to
\code{FALSE}.}

\item{tmn}{Calculate minimum temperature values (degrees Celsius)
and return it, \code{TRUE}. Defaults to \code{FALSE}.}

\item{tmx}{Calculate maximum temperature (degrees Celsius) and return it,
\code{TRUE}. Defaults to \code{FALSE}.}

\item{sunp}{Read sunshine, percent of maximum possible (percent of
day length) and return it, \code{TRUE}. Defaults to \code{FALSE}.}

\item{frs}{Read ground-frost records (number of days with ground-
frost per month) and return it, \code{TRUE}. Defaults to \code{FALSE}.}

\item{wnd}{Read 10 m wind speed (metres/second) and return it, \code{TRUE}.
Defaults to \code{FALSE}.}

\item{elv}{Read elevation (converted to metres) and return it, \code{TRUE}.
Defaults to \code{FALSE}.}

\item{x}{An optional local file path where \acronym{CRU} \acronym{CL} v.2.0
.dat.gz files are located.  If this is empty, the requested data will
automatically be downloaded from the server.}
}
\value{
A \link[data.table:data.table]{data.table::data.table} object of \acronym{CRU} \acronym{CL} v.
2.0 climatology elements.
}
\description{
Automates importing \acronym{CRU} \acronym{CL} v.2.0 climatology data and
from either the CRU server or local files and creates a \CRANpkg{data.table}
of the data. If requested, minimum and maximum temperature may also be
calculated as described in the data
\href{https://crudata.uea.ac.uk/cru/data/hrg/tmc/readme.txt}{readme.txt} file and
returned.
}
\section{Nomenclature and Units}{

\describe{
\item{pre}{precipitation (millimetres/month)}
\describe{
\item{cv}{cv of precipitation (percent)}
}
\item{rd0}{wet-days (number days with >0.1 millimetres rain per month)}
\item{tmp}{mean temperature (degrees Celsius)}
\item{dtr}{mean diurnal temperature range (degrees Celsius)}
\item{reh}{relative humidity (percent)}
\item{sunp}{sunshine (percent of maximum possible (percent of day length))}
\item{frs}{ground-frost (number of days with ground-frost per month)}
\item{wnd}{10 metre wind speed (metres/second)}
\item{elv}{elevation (automatically converted to metres from kilometres)}
}
For more information see the description of the data provided by
\acronym{CRU}, \url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/readme.txt}
}

\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
# Create a data frame of temperature from locally available files in the
# tempdir() directory.
library(fs)

f <- path(path_temp(), "grid_10min_tmp.dat.gz")

download.file(
  url = "https://crudata.uea.ac.uk/cru/data/hrg/tmc/grid_10min_tmp.dat.gz",
  destfile = f
)


cru_tmp <- read_cru_dt(tmp = TRUE, x = f)

cru_tmp

# or downloading directly from the CRU server

cru_tmp <- read_cru_dt(tmp = TRUE)

cru_tmp
\dontshow{\}) # examplesIf}
}
\references{
New, Mark, et al. "A high-resolution data set of surface climate
over global land areas." Climate research 21.1 (2002): 1-25.
\url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/new_et_al_10minute_climate_CR.pdf}
}
\seealso{
\link{read_cru_rast}.
}
\author{
Adam H. Sparks, \email{adamhsparks@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal-create_dt.R
\name{.drop_source_vars_dt}
\alias{.drop_source_vars_dt}
\title{Remove source vars if not requested}
\usage{
.drop_source_vars_dt(dt, vars)
}
\arguments{
\item{dt}{A data.table object of CRU data containing \code{tmp} and \code{dtr} in order
to calculate tmin and tmax.}

\item{vars}{A named logical vector.}
}
\value{
A data.table with the unrequested var columns removed.
}
\description{
If \code{tmin} or \code{tmax} are requested but \code{tmp} or \code{dtr} are not requested, this
function drops the unrequested vars.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCRUCLdata-package.R
\docType{package}
\name{getCRUCLdata-package}
\alias{getCRUCLdata}
\alias{getCRUCLdata-package}
\title{getCRUCLdata: 'CRU' 'CL' v. 2.0 Climatology Client}
\description{
Provides functions that automate downloading and importing University of East Anglia Climate Research Unit ('CRU') 'CL' v. 2.0 climatology data, facilitates the calculation of minimum temperature and maximum temperature and formats the data into a data.table object or a 'terra' 'SpatRaster' object. 'CRU' 'CL' v. 2.0 data are a gridded climatology of 1961-1990 monthly means released in 2002 and cover all land areas (excluding Antarctica) at 10 arc minutes (0.1666667 degree) resolution. For more information see the description of the data provided by the University of East Anglia Climate Research Unit, \url{https://crudata.uea.ac.uk/cru/data/hrg/tmc/readme.txt}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://codeberg.org/ropensci/getCRUCLdata}
  \item \url{https://docs.ropensci.org/getCRUCLdata/}
  \item Report bugs at \url{https://codeberg.org/ropensci/getCRUCLdata/issues}
}

}
\author{
\strong{Maintainer}: Adam H. Sparks \email{adamhsparks@gmail.com} (\href{https://orcid.org/0000-0002-0061-8359}{ORCID})

Authors:
\itemize{
  \item Adam H. Sparks \email{adamhsparks@gmail.com} (\href{https://orcid.org/0000-0002-0061-8359}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal-create_dt.R
\name{.add_derived_dt}
\alias{.add_derived_dt}
\title{Add derived temperature values for TMAX and TMIN}
\usage{
.add_derived_dt(dt, vars)
}
\arguments{
\item{dt}{A data.table object of CRU data containing \code{tmp} and \code{dtr} in order
to calculate tmin and tmax.}

\item{vars}{A named logical vector.}
}
\value{
A data.table with tmin and or tmax cols.
}
\description{
Add derived temperature values for TMAX and TMIN
}
